#----------------------------------------------------------------
#
# Mapmaker version 2.01 (Python version)  Last revised 08/26/05
#
# By John Trauger (some elements from version 1 witten by Tim Sargent)
#
# vorlon@vorlonagent.com
#
#----------------------------------------------------------------

import sys


class GameStuff:

# Keeps track of Game-wide variables and misc stuff that might otherwise
# be assigned to a global variable.  Also includes important reverse-
# lookup tables.  This structure is created once.
    
    currTurn = "0"
    currImpulse = "0"
    nSides = 2
    gameName = "Fog 4"
    counterPath = "/counters"
    ssdPath = "/ssds"
    mapPath = "/maps"
    sitrepPath = ""
    sitrepFileName = "fog4_sitrep.txt"
    validSides = {}
    validDirections = "ABCDEF"
    validSideClasses = "SBTM"
    validMinorClasses = "SFDPPFMTIBO"  
    validNeutralClasses = "SBMAPT"
    stackThreshold = 6
    unitArrange = ["n/a",[],[],[],[],[],[],[],[]]
    unitOffset = [[],[],[],[],[],[],[],[],[]]
    facingOffset = {"a":0,"b":1,"c":3,"d":4,"e":5,"f":7}
    positionList = {}
    classLookup = {}

class SideStuff:
# This structure is intantiated once for each distince side in a
# scenario plus one extra for "neutral" phenominon.
    
    sideName = "Federation"
    counterFolder = "fed/"
    desigLetter = "N"
    unitList = {}
    counterList = {}

# Info Boxes
    titleTextColor = "#000000"  # Black
    titleColor = "#C0C0C0"      # Gray
    bodyTextColor = "#000000"   # White
    bodyColor = "#FFFFFF"       # Black


class HexStuff:
# This structure holds information for each hex on the map that is
# occupied by one or more units: A list of the units in the hex
# (.unitList), the tooltip display information (.tooltips), and the
# info box display html (.infoBoxes). Each line of info box data gets
# its own entry in infoBoxes[].
    
    unitList = []
    tooltips = ""
    infoBoxes = []

    
class DataFields:
# Contains information telling the system how to parse a sitrep
# Each type of unit in DataStuff below needs the same set of
# information tracked for it: The list of designator(s) for that
# class (.desig), What fields will be read for that class (.read),
# What field(s) will go in the info box header (.boxTitle), what
# fields will go in the body of the info box (.boxData) and what
# fields will be placed in the tooltip for the hex(.tooltip)
    desig = "z"
    read = "unitID,unitClass,unitHex,unitSpd,unitEW,unitNotes"
    boxTitle = "unitID"
    boxData = "unitHex,unitSpd,unitEW"
    tooltip = "unitID,unitHex,unitSpd,unitEW"

class DataStuff:
    read = "unitID,unitClass,unitHex,unitSpd,unitEW,unitNotes"
    boxTitle = "unitID"
    boxData = "unitHex,unitSpd,unitEW"
    tooltip = "unitID,unitHex,unitSpd,unitEW"
    base = DataFields()
    ship = DataFields()
    pf = DataFields()
    shuttle = DataFields()
    drone = DataFields()
    plasma = DataFields()
    tb = DataFields()
    mine = DataFields()
    web = DataFields()


class UnitStuff:
# Stores all data for an individual unit.  Since properties
# can be added after the fact, not all possible properties
# are shown from the point of definition.

    unitID = ""
    ssdFn = ""
    hex = "0101"
    displayOffsetLeft = 0
    displayOffsetTop = 0
    counterSize = 20
    facing = "A"
    speed = "0"
    notes = ""
    infoBox = []
    owningSide = 0
    unitClass = "ca"
    ecm = "0"
    eccm = "0"
    isBackground = False
    isSmallUnit = False
    hitPointsSU = "0"
    damageTakenSU = "0"
    controllingUnitID = "NU1"
    ActiveESGs = False
    esgList = []
    isTractoring = False
    tractoredList = []
    isTractored = False
    tracotringUnitID = ""
    tooltipStuff = ""

# End of UnitStuff

class MapStuff:
    mType = "0"
    mColor = "black"
    mBackFN = "blackPlainMap.gif"
    mHeight = "1565"
    mWidth = "1277"
    mOutPath = ""
    mOutFileFormat = "blackMap.htm"
    mOutFilename = "blackMap.htm"
    mSSDClick = False
    mToolTips = False
    mInfoBoxes = False
    mLeadingZeroes = False
    
    leftOffset = 0
    topOffset = 0
    evenTopOffset = 0
    
    leftOneHex = 29
    downOneHex = 34

    bcTopOffset = -4
    bcLeftOffset = -4
    efTopOffset = -4
    efLeftOffset = -5

    clusterOffset = 7
    stackTopOffset = -2
    stackLeftOffset = 2

    #    newMapType is a string containing a number in string form, currently
    #    "0 or "1" where 0=plain, 1 = numbered hexes
    #    newMapColor is usually "white" or "black", indicating the map's background
    #       color.
    
    
    def setNewMap(mapRef):
        mapType = str(mapRef.mType)
   
        if (mapType == "0"):
            mapRef.mBackFN = mapRef.mColor + "PlainMap.gif"
            mapRef.mHeight = "1565"
            mapRef.mWidth = "1277"

            mapRef.leftOffset = 5
            mapRef.topOffset = -10
            mapRef.evenTopOffset = 17

        elif (mapType == "1"):
            mapRef.mBackFN = mapRef.mColor + "NumberedMap.gif"
            mapRef.mHeight = "1550"
            mapRef.mWidth = "1231"

            mapRef.leftOffset = -18
            mapRef.topOffset = -25
            mapRef.evenTopOffset = 16
        else :
            sys.stdout.write("SETNEWMAP ERROR : Invalid Map Type >")
            sys.stdout.write(mapType)
            sys.stdout.write("<")
            # error reporting goes here
        
# End of mapStuff


def iniRead(iniPath,iniFileName):
    
    ShowComments = False

    iniFile = open(iniPath + iniFileName,"r")
    inFile = iniFile.readlines()

    iniVars = []

    for inStuff in inFile:
        isError = False
        evalThis = True
        inStuff=inStuff.strip()
        inStuffSplit = inStuff.split("=")
        if (ShowComments):
            sys.stdout.write("Ini line: >" + inStuff + "<\n")
        
        if (inStuff == ""):
            outStuff = "No input"
            evalThis = False

        elif (inStuff[0] == "#"):
            outStuff = "Comment"
            evalThis = False
                 
        elif (len(inStuffSplit) < 2):
            outStuff = "IniRead - Bad Expression: >" + inStuff + "<"
            evalThis = False
            isError = True

        else:
            inVar = inStuffSplit[0].strip()
            inVal = inStuffSplit[1].strip()
            if (ShowComments):
                outStuff = "inVar = >" + inVar + "<\n" + "inVal = >" + inVal + "<\n"
                sys.stdout.write(outStuff)

            inValLow = inVal.lower()

            isNumeric = True
            for i in inVal:
                if (i not in "1234567890."):
                    isNumeric = False

            if (isNumeric):
                pass

            elif ((inValLow == "true") or (inValLow == "yes")):
                inVal = "True"
        
            elif ((inValLow == "false") or (inValLow == "no")):
                inVal = "False"
        
            elif ((inVal[0] == "'") and (inVal[len(inVal)-1] == '"')):
                inVal = inVal.replace("'",'"',1)

            elif ((inVal[0] == '"') and (inVal[len(inVal)-1] == "'")):
                inVal = inVal.replace('"',"'",1)

        if (evalThis):
            outStuff = [inVar, inVal]
            iniVars += [outStuff]

        if ((ShowComments) or (isError)):
            sys.stdout.write(outStuff)
            sys.stdout.write("\n\n")

    if (ShowComments):
        sys.stdout.write("iniVars = ")
        sys.stdout.write(iniVars)
        sys.stdout.write("\n" + "\n")

    return iniVars

# End of iniRead


def getCounter(unitType,sideInfo):

    global gameStuff

    unitType = unitType.lower()
    if (sideInfo.counterList == {}):
        counterList = iniRead(gameStuff.counterPath + sideInfo.counterFolder,"counters.ini")
        sideInfo.counterList = {"name":"counterList"}
        for i in counterList:
            sideInfo.counterList[i[0]] = i[1]
                
        if (not(sideInfo.counterList.has_key("default"))):
                sideInfo.counterList["default"] = "arrow"
    
    counterFile = ""
    cList = sideInfo.counterList
    utLen = len(unitType)

    compareList = [unitType]
    compareList += [unitType + "*"]
    compareList += [unitType[0:(utLen - 1)]]
    compareList += [compareList[2] + "*"]
    compareList += [unitType[0:(utLen - 2)]]
    compareList += [compareList[4] + "*"]
    compareList += ["default"]
        
    for i in compareList:
        if (cList.has_key(i)):
            counterFile = cList[i]
            compareChoice = i
            break

    if (compareChoice == "default"):
        pass
        #error condition

    if (counterFile == ""):
        counterFile = "arrow"
        #error condition

    return counterFile


# End of getCounter


def buildParentBox():
    pass
# End of buildParentBox


def buildMapHex():
    pass
# End of buildMapHex


def buildMapFile(mapInfo):

    global gameStuff, sideStuff, mapStuff

    mapInfo.setNewMap()
    mapBackground = mapInfo.mBackFN

    mapFN = mapInfo.mOutFileFormat
    mapFNSplit = mapFN.split("<")
    mapFN = mapFNSplit[0]

    for j in range(1,len(mapFNSplit)):
        fnPiece = mapFNSplit[j]
        fnInstr = fnPiece[0].lower()
        if (fnInstr == "t"):
            insertStuff = gameStuff.currTurn
            if ((mapInfo.mLeadingZeroes) and (len(insertStuff) == 1)):
                insertStuff = "0" + insertStuff
        elif (fnInstr == "i"):
            insertStuff = gameStuff.currImpulse
            if ((mapInfo.mLeadingZeroes) and (len(insertStuff) == 1)):
                insertStuff = "0" + insertStuff
        else:
            insertStuff = "+E+"
            # error condition.

        fnSplit = fnPiece.split(">")
        keeperStuff = ""
        for k in range(1,len(fnSplit)):
            if (k == 1) :
                keeperStuff += fnSplit[k]
            else:
                keeperStuff += ">" + fnSplit[k]

        mapFN += insertStuff + keeperStuff

    mapFile = open(mapFN,"w")

    mapFile.write('<html><head><title>'+ gameStuff.gameName)
    if ( not((gameStuff.currTurn == "0") or (gameStuff.currImpulse == "0"))):
        mapFile.write(' Turn:' + gameStuff.currTurn + ' Impulse:' + gameStuff.currImpulse)
    mapFile.write('</title>')
    mapFile.write('<META http-equiv=Content-Type content="text/html; charset=windows-1252">\n<STYLE>\n')
    for i in sideStuff:
        for j in ["left","right"]:
            mapFile.write('TD.' + i.desigLetter.lower() + 'Title' + j.title() + '{\n')
            mapFile.write('   FONT-SIZE: 12px; CURSOR: default; COLOR: ' + i.titleTextColor + ";")
            mapFile.write(' BACKGROUND-COLOR: '+ i.titleColor +'; TEXT-ALIGN: ' + j + '\n}\n')
            mapFile.write('TD.' + i.desigLetter.lower() + 'Body' + j.title() + '{\n')
            mapFile.write('   FONT-SIZE: 12px; CURSOR: default; COLOR: ' + i.bodyTextColor + ";")
            mapFile.write(' BACKGROUND-COLOR: '+ i.bodyColor +'; TEXT-ALIGN: ' + j + '\n}\n')
    mapFile.write('</STYLE>\n')
    mapFile.write('<SCRIPT LANGUAGE="JavaScript1.2" SRC="layers.js"></SCRIPT></head>\n')
    mapFile.write('<body bgcolor="#000000"><span style="position: absolute; left: 0; top: 0; z-index: 0"><img border="0" name="map" src="')
    mapFile.write(gameStuff.mapPath + mapBackground)
    mapFile.write('" width="' + mapInfo.mWidth + '" height="' + mapInfo.mHeight + '">')
    mapFile.write('</span>\n\n')

    
    ShowComments = False

    hexList = gameStuff.positionList.keys()
    hexList.sort()

    for j in hexList:
        hexData = gameStuff.positionList[j]
        unitsInHex = hexData.unitList

        hexCols = int(j[0:2])
        hexRows = int(j[2:4])

        if (ShowComments):
            sys.stdout.write("\n\nBUILDMAP : hex = " + j + " hexCols = ")
            sys.stdout.write(hexCols)
            sys.stdout.write(" hexRows = ")
            sys.stdout.write(hexRows)
            sys.stdout.write("\n\n Units in this hex:")
            sys.stdout.write(unitsInHex)

        nUnits = len(unitsInHex)
        

        if (nUnits > 1):
            unitsInHex.sort()
            unitsInHex.reverse()

        coarseLeftPos = hexCols * mapInfo.leftOneHex
        coarseTopPos = (hexRows * mapInfo.downOneHex) + mapInfo.topOffset
        
        coarseLeftPos += mapInfo.leftOffset
        
        if (hexCols % 2 == 0):
            coarseTopPos += mapInfo.evenTopOffset


        primeUnitFacing = ""
        nUnits = len(unitsInHex)

        for k in range(nUnits):
            sDesig = unitsInHex[k][0]
            foundSide = gameStuff.validSides.has_key(sDesig)

            if (foundSide) :
                if (ShowComments):
                    sys.stdout.write("\nBUILDMAP : nUnits = ")
                    sys.stdout.write(nUnits)
                    sys.stdout.write(" k = ")
                    sys.stdout.write(k)
                    sys.stdout.write(" unitsInHex[k] = >" + unitsInHex[k])
                    sys.stdout.write("<\n")

                sideInfo = gameStuff.validSides[sDesig]
                unitInfo = sideInfo.unitList[unitsInHex[k]]

                sDesigLower = sDesig.lower()
                unitFacing = unitInfo.facing
                if (unitFacing not in gameStuff.validDirections):
                    unitFacing = "A"
                    
                unitFacing = unitFacing.lower()

                if ((unitFacing == "b") or (unitFacing == "c")):
                    unitLeftOffset = mapInfo.bcLeftOffset
                    unitTopOffset = mapInfo.bcTopOffset
                    unitInfo.counterSize = "29"
                elif ((unitFacing == "e") or (unitFacing == "f")):
                    unitLeftOffset = mapInfo.efLeftOffset
                    unitTopOffset = mapInfo.efTopOffset
                    unitInfo.counterSize = "29"
                else :
                    unitLeftOffset = 0
                    unitTopOffset = 0
                    unitInfo.counterSize = "20"
 
                if (nUnits == 1):
                    unitArrangeLeft = 0
                    unitArrangeTop = 0
                elif (nUnits >= gameStuff.stackThreshold):
                    unitArrangeLeft = (mapInfo.stackLeftOffset * k) - nUnits/2
                    unitArrangeTop = (mapInfo.stackTopOffset * k) + nUnits/2
                else:
                    hexArrange = gameStuff.unitArrange[nUnits]
        
                    if (ShowComments):
                        sys.stdout.write("\nBUILDMAP : hexArrange = ")
                        sys.stdout.write(hexArrange)
                        sys.stdout.write("\n")
                        
                    if (primeUnitFacing == ""):
                        primeUnitFacing = unitFacing
                        primeUnitOffset = gameStuff.facingOffset[primeUnitFacing]

                        if (ShowComments):
                            sys.stdout.write("BUILDMAP : primeUnitFacing = >" + primeUnitFacing)
                            sys.stdout.write("< primeUnitOffset = >")
                            sys.stdout.write(primeUnitOffset)
                            sys.stdout.write("<\n")


                    unitArrange = (hexArrange[k] + primeUnitOffset) % 8
                    if (unitArrange == 0):
                        unitArrange = 8
                    unitArrange = gameStuff.unitOffset[unitArrange]

                    if (ShowComments):
                        sys.stdout.write("BUILDMAP : unitArrange = " + unitArrange + "\n")

                    unitArrangeLeft = int(unitArrange.split(",")[1]) * mapInfo.clusterOffset
                    unitArrangeTop = int(unitArrange.split(",")[0]) * mapInfo.clusterOffset

                        
                finalUnitLeft = coarseLeftPos + unitLeftOffset + unitArrangeLeft
                finalUnitTop = coarseTopPos + unitTopOffset + unitArrangeTop

                unitInfo.displayOffsetLeft = finalUnitLeft
                unitInfo.displayOffsetTop = finalUnitTop

                        

#
# This space for processing effects like ESGs, tractor beams
# and for placing background objects like terrain.
#                
    for j in hexList:
        hexData = gameStuff.positionList[j]
        unitsInHex = hexData.unitList
        
#        if (ShowComments):
#            sys.stdout.write("BUILDMAP : hexDesc = ")
#            sys.stdout.write(hexDesc)
#            sys.stdout.write("<\n\n")

        nUnits = len(unitsInHex)

        if (nUnits > 1):
            unitsInHex.sort()
            unitsInHex.reverse()
        
        for k in range(nUnits):
            sDesig = unitsInHex[k][0]
            foundSide = gameStuff.validSides.has_key(sDesig)
            if (foundSide) :
                sideInfo = gameStuff.validSides[sDesig]
                unitInfo = sideInfo.unitList[unitsInHex[k]]
                unitFacing = unitInfo.facing
       
                counterFN = getCounter(unitInfo.unitClass,sideInfo) + "-" + unitFacing + ".gif"
                counterToUse = gameStuff.counterPath + sideInfo.counterFolder + counterFN.lower()
                finalUnitLeft = unitInfo.displayOffsetLeft
                finalUnitTop = unitInfo.displayOffsetTop
                counterDim =unitInfo.counterSize

                if ((mapInfo.mSSDClick) and (unitInfo.ssdFn)):
                    ssdPath = gameStuff.ssdPath + unitInfo.ssdFn
                    clickStuff = ' onClick = "displaySSD(' + "'" + ssdPath  + "'" + ')" '
                else:
                    clickStuff = ''
                
                if (mapInfo.mToolTips):
                    hexStuff = 'alt="' + hexData.tooltips + '" '
                else:
                    hexStuff = ''

                if (mapInfo.mInfoBoxes):
                    if (k == 0):
                        layerID = ""
                        layerIDSplit = unitsInHex[k].split(".")
                        
                        for m in layerIDSplit:
                            layerID += m
                        
                        finalUnitLeft = unitInfo.displayOffsetLeft
                        finalUnitTop = unitInfo.displayOffsetTop

                        if (finalUnitLeft < 85):
                            boxLeft = finalUnitLeft + 40
                        else:
                            boxLeft = finalUnitLeft - 80
                            
                        boxTop = len(hexData.infoBoxes)

                        boxTopHigh = finalUnitTop - (boxTop * 15)
                        boxTopLow = finalUnitTop - (boxTop * 8)
                        boxTopBottom = finalUnitTop + (boxTop * 8)

                        if (boxTopLow < 10):
                            boxTop = finalUnitTop - 10
                        elif (boxTopBottom > mapInfo.mHeight):
                            boxTop = boxTopHigh
                        else:
                            boxTop = boxTopLow
                            
                        infoBoxStuff =  '<DIV NAME="'+layerID+'" ID="'+layerID+'" STYLE="position:absolute; left:' + str(boxLeft)+'; top:'+ str(boxTop) + '; visibility:hidden; z-index:4">\n'
                        infoBoxStuff += '<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0" width = "74">\n'
                        for m in hexData.infoBoxes:
                            infoBoxStuff += m
                        infoBoxStuff += '</TABLE></DIV>\n'

                        if (ShowComments):
                            sys.stdout.write("BUILDMAP : Units in hex[0] = ")
                            sys.stdout.write(unitsInHex[0])
                            sys.stdout.write("\n")
                    
                    infoStuff = 'onMouseOver="JavaScript:showLayer(\'' + layerID + '\');" onMouseOut="JavaScript:hideLayer(\'' + layerID + '\');"'
                else:
                    infoStuff = ''
                    infoBoxStuff = ''

 
                mapLine = '<span style="position: absolute; left: ' + str(finalUnitLeft)
                mapLine += '; top: ' + str(finalUnitTop)
                mapLine += '; z-index: 2"><img border="0" name="ship" src="' + counterToUse +'" '
                mapLine += hexStuff
                mapLine += 'width="' + counterDim + '" height="' + counterDim + '"'
                mapLine += clickStuff
                mapLine += infoStuff
                mapLine += '></span>\n'
                
                mapFile.write(mapLine)

        mapFile.write(infoBoxStuff)

        if (ShowComments):
            sys.stdout.write("\nBUILDMAP : infoBoxStuff = ")
            sys.stdout.write(infoBoxStuff)
            sys.stdout.write("\n\n")
                    

    mapFile.write('</body></html>')
    mapFile.close()

# End of buildMapFile


def getUnitClass(unitIDSplit):

    uClass = ""
    if (len(unitIDSplit) == 1):
        uID = unitIDSplit[0][1:]
        for i in uID:
            if (i.lower() in "abcdefghijklmnopqrstuvwxyz"):
                uClass += i
    else:
        for i in unitIDSplit[len(unitIDSplit) - 1]:
            if (i.lower() in "abcdefghijklmnopqrstuvwxyz"):
                uClass += i
        uClass = "." +uClass

    return uClass.lower()


# End of buildMapFile


def validSitrepLine(sLine):

    ShowComments = False

    if (ShowComments):
        sys.stdout.write("VALIDSITREPLINE: sLine = >" + sLine + "<\n")

    isValid = False

    if (sLine == ""):
        pass
    elif (sLine[0][0] == "#"):
        pass
    else:
        sLineSplit = sLine.split()
        unitID = sLineSplit[0]
        unitIDSplit = unitID.split(".")

        uClass = getUnitClass(unitIDSplit)
        if (uClass == ".h"):
            unitIDSplit = unitIDSplit[0:(len(unitIDSplit)-1)]
            uClass = getUnitClass(unitIDSplit)
        

        if (ShowComments):
            sys.stdout.write("VALIDSITREPLINE: uClass = >" + uClass + "<\n")
    

        dataRef = ""
        if (GameStuff.classLookup.has_key(uClass)):
            dataRef = GameStuff.classLookup[uClass]
            if (ShowComments):
                sys.stdout.write("VALIDSITREPLINE: dataRef.read = >" + dataRef.read + "<\n")
        else:
             if (ShowComments):
                sys.stdout.write("VALIDSITREPLINE: No Class match. dataRef = >" + dataRef + "<\n")
       

        if (dataRef == ""):
            nFields = 1000
        else:
            readSplit = dataRef.read.split(",")
            nFields = len(readSplit)
            if ("unitnotes" in dataRef.read.lower()):
                nFields -= 1

        if (ShowComments):
            sys.stdout.write("VALIDSITREPLINE: nFields = >")
            sys.stdout.write(nFields)
            sys.stdout.write("< len(sLineSplit) = >")
            sys.stdout.write(len(sLineSplit))
            sys.stdout.write("<\n")

        if(dataRef == ""):
            pass
        elif (len(sLineSplit) < nFields):
            pass
        else:
            if (ShowComments):
                sys.stdout.write("VALIDSITREPLINE: readSplit = >")
                sys.stdout.write(readSplit)
                sys.stdout.write("<\n")

            isValid = True
            fCount = -1
            for i in readSplit:
                fCount += 1
                if (i == "unitID"):
                    sDesig = unitID[0]
                    uDesig = unitID[1]
                    if (not (gameStuff.validSides.has_key(sDesig))):
                        isValid = False
                    if (not (uDesig in gameStuff.validSideClasses)):
                        isValid = False
                        
                elif (i == "unitClass"):
                    pass
                
                elif (i == "unitHex"):
                    uHex = sLineSplit[fCount]
                    uHexLen = len(uHex)
                    if (not ((uHexLen == 4) or (uHexLen == 5))):
                        isValid = False
                    else:
                        hValid = True
                        uHex = uHex[0:4]
                        for i in uHex:
                            hValid = hValid and (i in "0123456789")
                        isValid = isValid and hValid
                    
                elif (i == "unitName"):
                    uName = sLineSplit[fCount]
                    uNameSplit = uName.split('"')
                    if (len(uNameSplit) == 3):
                        if ( not((uNameSplit[0] == "") and (uNameSplit[0] == ""))):
                            isValid = False
                    else:
                        isValid = False

                elif (i == "unitSpd"):
                    uSpeed = sLineSplit[fCount]
                    uSpeedLen = len(uSpeed)
                    if ( not (uSpeedLen in [1,2,3])):
                        isValid = False
                    else:
                        sValid = True
                        for i in uSpeed:
                            sValid = sValid and (i in "-0123456789")
                        isValid = isValid and sValid

                elif (i == "unitStrength"):
                    uStrength = sLineSplit[fCount]
                    uStrengthLen = len(uStrength)
                    if ( not (uStrengthLen in [1,2])):
                        isValid = False
                    else:
                        sValid = True
                        for i in uStrength:
                            sValid = sValid and (i in "0123456789")
                        isValid = isValid and sValid

                elif (i == "unitDamage"):
                    uDamage = sLineSplit[fCount]
                    uDamageLen = len(uDamage)
                    if ( not (uDamageLen in [1,2,3])):
                        isValid = False
                    else:
                        sValid = True
                        for i in uDamage:
                            sValid = sValid and (i in "0123456789")
                        isValid = isValid and sValid

                elif (i == "unitWarhead"):
                    uWarhead = sLineSplit[fCount]
                    uWarheadLen = len(uWarhead)
                    if ( not (uWarheadLen in [1,2,3])):
                        isValid = False
                    else:
                        sValid = True
                        for i in uWarhead:
                            sValid = sValid and (i in "0123456789")
                        isValid = isValid and sValid

                elif (i == "unitEW"):
                    uEW = sLineSplit[fCount]
                    uEWSplit = uEW.split("/")
                    if (not(len(uEWSplit) == 2)):
                        isValid = False

                elif (i == "unitLaunch"):
                    uLaunch = sLineSplit[fCount]
                    uLaunchSplit = uLaunch.split(".")
                    if (not(len(uLaunchSplit) == 2)):
                        isValid = False

                elif (i == "unitLaid"):
                    uLaid = sLineSplit[fCount]
                    uLaidSplit = uLaid.split(".")
                    if (not(len(uLaidSplit) == 2)):
                        isValid = False

                elif (i == "unitCast"):
                    uCast = sLineSplit[fCount]
                    uCastSplit = uCast.split(".")
                    if (not(len(uCastSplit) == 2)):
                        isValid = False
                        
                elif (i == "unitNotes"):
                    pass

    return isValid

    
# End of validSitrepLine


def StoreUnitInfo(fieldSplit,dataSplit,unitRef):

    nFields = len(fieldSplit)
    nData = len(dataSplit)
    nDiff = nFields - nData
    if(nDiff > 0):
        for i in range(nDiff):
            dataSplit = dataSplit + [""]

    for i in range(nFields):
        currField = fieldSplit[i]
        currData = dataSplit[i]

        if (currField == "unitID"):
            unitRef.unitID = currData

        elif (currField == "unitClass"):
            unitRef.unitClass = currData
                
        elif (currField == "unitName"):
            unitRef.name = currData
                
        elif (currField == "unitHex"):
            unitRef.hex = currData
                
        elif (currField == "unitSpd"):
            unitRef.speed = currData
                
        elif (currField == "unitStrength"):
            unitRef.strength = currData
                
        elif (currField == "unitDamage"):
            unitRef.damage = currData
                
        elif (currField == "unitWarhead"):
            unitRef.warhead = currData
                
        elif (currField == "unitEW"):
            ewSplit = currData.split("/")
            unitRef.ecm = ewSplit[0]
            unitRef.eccm = ewSplit[1]

        elif (currField == "unitLaunch"):
            unitRef.launch = currData

        elif (currField == "unitLaid"):
            unitRef.launch = currData
                
        elif (currField == "unitCast"):
            unitRef.launch = currData
                
        elif (currField == "unitNotes"):
            unitRef.notes = currData
                
 
# End of StoreUnitInfo


def buildTipLines(fieldSplit,unitRef):

    outLines = ""
    
    for i in fieldSplit:
        skipField = False
        if (i == "unitID"):
            dData = unitRef.unitID

        elif (i == "unitClass"):
            dData = " Class:" + unitRef.hex
                
        elif (i == "unitName"):
            dData = " Name:" + unitRef.hex
                
        elif (i == "unitHex"):
            dData = " Hex:" + unitRef.hex
                
        elif (i == "unitSpd"):
            dData = " Speed:" + unitRef.speed
                
        elif (i == "unitStrength"):
            dData = " Strength:" + unitRef.strength
                
        elif (i == "unitDamage"):
            dData = " Damage:" + unitRef.damage
                
        elif (i == "unitWarhead"):
            dData = " Warhead:" + unitRef.warhead
                
        elif (i == "unitEW"):
            dData = " EW:" + unitRef.ecm + "/" + unitRef.eccm
                
        elif (i == "unitLaunch"):
            dData = " Launched:" + unitRef.launch

        elif (i == "unitLaid"):
            dData = " Laid:" + unitRef.launch

        elif (i == "unitCast"):
            dData = " Cast:" + unitRef.launch
                
        elif (i == "unitNotes"):
            if (unitRef.unitNotes == ""):
                skipField = True
            else:
                dData = "\n  Notes:" + unitRef.notes
                
        else:
            skipField = True

        if (not(skipField)):
            outLines += dData
    
    return outLines


# End of buildTipLines


def buildBoxLines(fieldSplit,unitRef,widthStuff,sideLetter,lineType):

    outLines = ""
    wLength = len(widthStuff)
    
    for i in fieldSplit:
        outLines += '<TR>'
        skipField = False
        if (i == "unitID"):
            dData = [unitRef.unitID]
            if(wLength == 2):
                dData = ["Unit:"] + dData

        elif (i == "unitClass"):
            dData = [unitRef.unitClass]
            if(wLength == 2):
                dData = ["Class:"] + dData

        elif (i == "unitName"):
            dData = [unitRef.name]
            if(wLength == 2):
                dData = ["Name:"] + dData
                
        elif (i == "unitHex"):
            dData = [unitRef.hex]
            if(wLength == 2):
                dData = ["Hex:"] + dData
                
        elif (i == "unitSpd"):
            dData = [unitRef.speed]
            if(wLength == 2):
                dData = ["Speed:"] + dData
                
        elif (i == "unitStrength"):
            dData = [unitRef.strength]
            if(wLength == 2):
                dData = ["Strength:"] + dData
                
        elif (i == "unitDamage"):
            dData = [unitRef.damage]
            if(wLength == 2):
                dData = ["Damage:"] + dData
                
        elif (i == "unitWarhead"):
            dData = [unitRef.warhead]
            if(wLength == 2):
                dData = ["Warhead:"] + dData
                
        elif (i == "unitEW"):
            dData = [unitRef.ecm + "/" + unitRef.eccm]
            if(wLength == 2):
                dData = ["EW:"] + dData
                
        elif (i == "unitLaunch"):
            dData = [unitRef.launch]
            if(wLength == 2):
                dData = ["Launched:"] + dData

        elif (i == "unitLaid"):
            dData = [unitRef.launch]
            if(wLength == 2):
                dData = ["Laid:"] + dData

        elif (i == "unitCast"):
            dData = [unitRef.launch]
            if(wLength == 2):
                dData = ["Cast:"] + dData

        elif (i == "unitNotes"):
            dData = [unitRef.notes]
            if(dData == [""]):
                skipField = True
            else:
                if(wLength == 2):
                    dData = ["Notes:"] + dData
                
        else:
            skipField = True

        if (not(skipField)):
            for j in range(wLength):
                if (j == 0):
                    outLines += '   <TD class=' + sideLetter.lower() + lineType.title() + 'Left'
                    if (wLength == 1):
                        outLines += ' colspan="2" '
                else:
                    outLines += '   <TD class=' + sideLetter.lower() + lineType.title() + 'Right'

                outLines += ' noWrap >'
                outLines += dData[j] + '</TD>\n'
    
        outLines += "</TR>\n"

    return outLines


# End of buildBoxLines


def buildUnitBox(whichSide,unitID,unitRef,dataRef):

    ShowComments = False

    tipSplit = dataRef.tooltip.split(",")
    unitRef.tooltipStuff = buildTipLines(tipSplit,unitRef)

    tempBoxStuff = ""
    titleSplit = dataRef.boxTitle.split(",")
    tempBoxStuff += buildBoxLines(titleSplit,unitRef,["100"],whichSide.desigLetter,"Title")

    if (ShowComments):
        sys.stdout.write("BUILDUNITBOX: tempBoxStuff = >" + tempBoxStuff + "<\n")
    
    dataSplit = dataRef.boxData.split(",")
    tempBoxStuff += buildBoxLines(dataSplit,unitRef,["40","60"],whichSide.desigLetter,"Body")

    unitRef.infoBox = tempBoxStuff


# End of buildUnitBox


def readSitrep():

    global gameStuff, sideStuff, mapStuff

    ShowComments = False

    if(ShowComments):
        sys.stdout.write("READSITREP: sitrep path and filename = " + gameStuff.sitrepPath + gameStuff.sitrepFileName)

    sitrepFile = open((gameStuff.sitrepPath + gameStuff.sitrepFileName),"r")
    sitrep = sitrepFile.readlines()

    for sitrepLine in sitrep:

        sitrepLine = sitrepLine.strip()
        
        if (ShowComments):
            sys.stdout.write("READSITREP: sitrep line = >" + sitrepLine + "<\n")

        if ("Information for" in sitrepLine):
            turnImp = sitrepLine.split("Information for")[1].strip()
            tiSplit = turnImp.split(".")
            gameStuff.currTurn = tiSplit[0]
            gameStuff.currImpulse = tiSplit[1]

        if (validSitrepLine(sitrepLine)) :
            sideDesig = sitrepLine[0]
            whichSide = gameStuff.validSides[sideDesig]
            sitrepStuff = sitrepLine.split()

            if (ShowComments):
                sys.stdout.write("READSITREP: This line is good.\n")
                sys.stdout.write("READSITREP: sideDesig = >" + sideDesig + "<\n")
                sys.stdout.write("READSITREP: sitrep split = ")
                sys.stdout.write(sitrepStuff)
                sys.stdout.write("\n")

            unitID = sitrepStuff[0]
            unitType = sitrepStuff[1]
            unitHex = sitrepStuff[2]
            unitSpeed = sitrepStuff[3]
            unitEW = sitrepStuff[4]
            unitNotes = ""
            roughNotes = ""

            if ("*" in unitID) :
                unitID = unitID.split("*")[0]

            unitHexOnly = unitHex[0:4]
            unitDir = unitHex[4]

            unitEWSplit = unitEW.split("/")
            unitECM = unitEWSplit[0]
            unitECCM = unitEWSplit[1]

            if (len(sitrepStuff) > 5) :
                roughNotes = sitrepStuff[5:]
                for j in roughNotes :
                    unitNotes += " " + j
                unitNotes = unitNotes.strip()

            if (not whichSide.unitList.has_key(unitID)):
                    whichSide.unitList[unitID] = UnitStuff()

            if (ShowComments):
                sys.stdout.write("READSITREP: unitID = >" + unitID + "<\n")

            unitIDSplit = unitID.split(".")

            uClass = getUnitClass(unitIDSplit)
            if (uClass == ".h"):
                unitIDSplit = unitIDSplit[0:(len(unitIDSplit)-1)]
                uClass = getUnitClass(unitIDSplit)

            if (ShowComments):
                sys.stdout.write("READSITREP: uClass = >" + uClass + "<\n")

            dataRef = ""
            if (GameStuff.classLookup.has_key(uClass)):
                dataRef = GameStuff.classLookup[uClass]
                if (ShowComments):
                    sys.stdout.write("READSITREP: dataRef.read = >" + dataRef.read + "<\n")
            else:
                if (ShowComments):
                    sys.stdout.write("READSITREP: No Class match. dataRef = >" + dataRef + "<\n")


            storeSplit = dataRef.read.split(",")
            StoreUnitInfo(storeSplit,sitrepStuff,dataRef)

            whichSide.unitList[unitID].unitID = unitID
            whichSide.unitList[unitID].hex = unitHexOnly
            whichSide.unitList[unitID].facing = unitDir
            whichSide.unitList[unitID].speed = unitSpeed
            whichSide.unitList[unitID].notes = unitNotes
            whichSide.unitList[unitID].unitClass = unitType
            whichSide.unitList[unitID].ecm = unitECM
            whichSide.unitList[unitID].eccm = unitECCM

            buildUnitBox(whichSide,unitID,whichSide.unitList[unitID],dataRef)

            if (not (gameStuff.positionList.has_key(unitHexOnly))):
                gameStuff.positionList[unitHexOnly] = HexStuff()

            hexStuff = gameStuff.positionList[unitHexOnly]
            if (hexStuff.unitList == []):
                hexStuff.unitList = [unitID]
                hexStuff.tooltips = whichSide.unitList[unitID].tooltipStuff
                hexStuff.infoBoxes = whichSide.unitList[unitID].infoBox
                
            else:
                hexStuff.unitList += [unitID]
                hexStuff.tooltips += '\n' + whichSide.unitList[unitID].tooltipStuff
                hexStuff.infoBoxes += whichSide.unitList[unitID].infoBox


# End of readSitrep


def addEntry(listName,propName,propIndex,propVal):

    global sideStuff,mapStuff,gameStuff
    ShowComments = False
        
    stuffName = listName + "Stuff"
    entryName = listName + "Entries"

    if (ShowComments):
        sys.stdout.write("ADDENTRY listName = >")
        sys.stdout.write(listName)
        sys.stdout.write("< propName = >")
        sys.stdout.write(propName)
        sys.stdout.write("< propIndex = >")
        sys.stdout.write(propIndex)
        sys.stdout.write("< propVal = >")
        sys.stdout.write(propVal)
        sys.stdout.write("<\n")

    propIndex = int(propIndex)

    if (listName == "map"):
        if (mapStuff == []):

            if (ShowComments):
                sys.stdout.write("ADDENTRY Empty list detected<\n")

            for i in range(propIndex+1):
                mapStuff += [MapStuff()]
                    
        elif (propIndex >= len(mapStuff)):
            for i in range((len(mapStuff)),propIndex+1):
                mapStuff += [MapStuff()]
        if (propName == "numbered"):
                propName = "mType"
        elif (propName == "backColor"):
            propName = "mColor"
        elif (propName == "nameFormat"):
            propName = "mOutFileFormat"
        elif (propName == "toolTips"):
            propName = "mToolTips"
        elif (propName == "infoBoxes"):
            propName = "mInfoBoxes"
        elif (propName == "ssdClick"):
            propName = "mSSDClick"
        elif (propName == "leadingZeroes"):
            propName = "mLeadingZeroes"

        if (ShowComments):
            sys.stdout.write("ADDENTRY mapStuff = ")
            sys.stdout.write(mapStuff)
            sys.stdout.write(" propName = " + propName)
            sys.stdout.write("\n")
            sys.stdout.write("ADDENTRY opStr = >mapStuff[propIndex]." + propName + "=" + propVal+"<\n")
        
        opStr = compile("mapStuff[propIndex]." + propName + "=" + propVal,"<string>","single")
        eval(opStr)
            
    elif (listName == "side"):
        if (sideStuff == []):

            if (ShowComments):
                 sys.stdout.write("ADDENTRY Empty list detected<\n")

            for i in range(propIndex+1):
                sideStuff += [SideStuff()]
                sideStuff[i].unitList = {i:i}
        elif (propIndex >= len(sideStuff)):
            for i in range((len(sideStuff)),propIndex+1):
                sideStuff += [SideStuff()]
                sideStuff[i].unitList = {i:i}

        if (ShowComments):
            sys.stdout.write("ADDENTRY sideStuff = ")
            sys.stdout.write(sideStuff)
            sys.stdout.write("\n")
            sys.stdout.write("ADDENTRY opStr = >sideStuff[propIndex]." + propName + "=" + propVal+"<\n")
                
        opStr = compile("sideStuff[propIndex]." + propName + "=" + propVal,"<string>","single")
        eval(opStr)

    elif (listName == "unit"):
        if (sideStuff == []):

            if (ShowComments):
                sys.stdout.write("ADDENTRY Empty list detected<\n")

            for i in range(propIndex+1):
                sideStuff += [SideStuff()]
                sideStuff[i].unitList = {i:i}
        elif (propIndex >= len(sideStuff)):
            for i in range((len(sideStuff)),propIndex+1):
                sideStuff += [SideStuff()]
                sideStuff[i].unitList = {i:i}

        if (ShowComments):
            sys.stdout.write("ADDENTRY sideStuff[")
            sys.stdout.write(propIndex)
            sys.stdout.write("].unitList.keys() = ")
            sys.stdout.write(sideStuff[propIndex].unitList.keys)
            sys.stdout.write("\n")

        propVal = propVal.split('"')[1]
        currentStuff = propVal.split(",")
        currentUnitID = currentStuff[0].strip()
        currentUnitFn = currentStuff[1].strip()

        if (not sideStuff[propIndex].unitList.has_key(currentUnitID)):
            sideStuff[propIndex].unitList[currentUnitID] = UnitStuff()

        sideStuff[propIndex].unitList[currentUnitID].ssdFn = currentUnitFn

        sDesig = currentUnitID[0]
        if (gameStuff.validSides == {}):
            gameStuff.validSides = {sDesig : sideStuff[propIndex]}
        elif (not (gameStuff.validSides.has_key(sDesig))):
            gameStuff.validSides[sDesig] = sideStuff[propIndex]

        if (ShowComments):
            sys.stdout.write("ADDENTRY sideStuff[")
            sys.stdout.write(propIndex)
            sys.stdout.write("].unitList[" + currentUnitID +"] = " + currentUnitFn + "<\n")

    elif (listName == "game"):
        if (propName[0:4] == "unit"):
            propIndex = propName[len(propName)-1]
            propName = propName[0:len(propName)-1]
            opStr = compile("gameStuff." + propName + "[" + propIndex + "]" + "=" + propVal,"<string>","single")
        else:
            opStr = compile("gameStuff." + propName + "=" + propVal,"<string>","single")
                
        eval(opStr)
        
    elif (listName == "fields"):
        propNameSplit = propName.split(".")
        if (len(propNameSplit) == 2):
            if (propNameSplit[1].lower() == "desig"):
                propVal = propVal.lower()

                if (ShowComments):
                    sys.stdout.write("ADDENTRY propNameSplit = >")
                    sys.stdout.write(propNameSplit)
                    sys.stdout.write("<\n")

                propValSplit = propVal.split('"')[1].split(",")
                for uDesig in propValSplit:
                    if(GameStuff.classLookup == {}):
                        opStr = compile("GameStuff.classLookup = {uDesig.lower():DataStuff." + propNameSplit[0] + "}","<string>","single")
                    else:
                        opStr = compile("GameStuff.classLookup[uDesig.lower()] = DataStuff." + propNameSplit[0],"<string>","single")

                    eval(opStr)
        
        opStr = compile("DataStuff." + propName + "=" + propVal,"<string>","single")
        eval(opStr)

# End of addentry

             
def mapSetup():

    global True, False
    True = 1
    False = 0
    ShowComments = False
    
    global gameStuff, sideStuff, mapStuff

    gameStuff = GameStuff()
    sideStuff = []
    mapStuff = []
    
    errors = ""

    iniStuff = iniRead("","mapmaker.ini")

    for iniEntry in iniStuff:

        processThis = False
        errorMessage = False
        varName = iniEntry[0].strip()
        varInfo = iniEntry[1].strip()

        if (ShowComments):
            sys.stdout.write("iniEntry: >")
            sys.stdout.write(iniEntry)
            sys.stdout.write("< varName: >")
            sys.stdout.write(varName)
            sys.stdout.write("< varInfo: >")
            sys.stdout.write(varInfo)
            sys.stdout.write("<\n")

        if ("." not in varName):
            varName = "global." + varName

        varNameSplit1 = varName.split(".")
        objName = varNameSplit1[0].lower()
        varName = varNameSplit1[1]
        if (len(varNameSplit1) > 2):
            varNameTemp = varNameSplit1[2:]
            for iii in varNameTemp:
                varName += "." + iii
            
        varNameLow = varName.lower()

        if ((objName == "game") or(objName == "fields")):
            splitNumber = "0"
        elif (objName == "global"):
            pass
        else :
            splitNumber = objName[len(objName) - 1]
            objName = objName[0 : len(objName) - 1]

        if (ShowComments):
            sys.stdout.write("MAPSETUP objName: >")
            sys.stdout.write(objName)
            sys.stdout.write("<\n")

        if (objName == "side") :
            if (varNameLow == "ssdfn") :
                objName = "unit"

        if (objName == "global") :
            sideStr = str(varInfo)
            opStr = compile(("global " + varName + "; " + varName + "=" + str(varInfo)),"","single")
            eval(opStr)
        else :
            addEntry(objName,varName,splitNumber,varInfo)
                    
        if (ShowComments):
            sys.stdout.write("MAPSETUP Ini line: >")
            sys.stdout.write(iniEntry)
            sys.stdout.write("<\n")

    if (ShowComments):
        for i in range(gameStuff.nSides+1) :
            sys.stdout.write("MAPSETUP sideStuff[")
            sys.stdout.write(i)
            sys.stdout.write("] keys:")
            sys.stdout.write(sideStuff[i].unitList.keys())
            sys.stdout.write("\n\n")
    
    readSitrep()

    if (ShowComments):
        for i in range(gameStuff.nSides+1) :
            sys.stdout.write("MAPSETUP sideStuff[")
            sys.stdout.write(i)
            sys.stdout.write("] keys:")
            sys.stdout.write(sideStuff[i].unitList.keys())
            sys.stdout.write("\n\n")

    for i in mapStuff :
        buildMapFile(i)

    if (errors):
        sys.stdout.write("<\n")
        sys.stdout.write("Errors: ")
        sys.stdout.write(errors)
        sys.stdout.write("<\n")

# End of mapSetup
            

mapSetup()
